//----------------------------------------------------------------------
// File : AsCamera.cpp
// Desc : Asura Camera For Demo Application
// Copyright(c) Project Asura All right reserved.
//----------------------------------------------------------------------

//----------------------------------------------------------------------
// History
//----------------------------------------------------------------------
// <---> 2011/04/16 [Pocol] VK쐬
// <End>


//----------------------------------------------------------------------
// Includes
//----------------------------------------------------------------------
#include "asCamera.h"

//----------------------------------------------------------------------
// Using Statements
//----------------------------------------------------------------------
using namespace Asura;
using namespace Asura::Math;


namespace Asura {
namespace Graphics {

///---------------------------------------------------------------------
///<summary>
///ݒ菈
///</summary>
///---------------------------------------------------------------------
Camera::Camera()
{
    mType = CAMERA_TYPE_THIRD_PERSON;

    mPosition = Vector3( 0.0f, 0.0f, 1.0f );
    mDirection = Vector3( 0.0f, 0.0f, 0.0f );
    mUpVector = Vector3( 0.0f, 1.0f, 0.0f );

    mAngle = Vector2( 0.0f, 0.0f );
    mDistance = 1.0f;

    std::memset( &mLeft, 0, sizeof( Camera::MouseState ) );
    std::memset( &mRight, 0, sizeof( Camera::MouseState ) );
    std::memset( &mMiddle, 0, sizeof( Camera::MouseState ) );
   
    mView.Identity();
}

///---------------------------------------------------------------------
///<summary>
///fXgN^
///</summary>
///---------------------------------------------------------------------
Camera::~Camera()
{
    /* DO_NOTHING */
}

///---------------------------------------------------------------------
///<summary>
///J^Cvݒ肷
///</summary>
///<param name="type">J^Cv</param>
///---------------------------------------------------------------------
void Camera::SetType( int type )
{
    mType = type;
}

///---------------------------------------------------------------------
///<summary>
///Jʒuݒ肷
///</summary>
///<param name="position">Jʒu</param>
///---------------------------------------------------------------------
void Camera::SetPosition( Vector3 position )
{
    if ( mType == CAMERA_TYPE_THIRD_PERSON )
    {
        float dist = mPosition.Length();
        mDistance = dist;

        Vector3 xAxis( 1.0f, 0.0f, 0.0f );
        Vector3 yAxis( 0.0f, 1.0f, 0.0f );

        Vector3 normPos = position;
        normPos.Normalize();
        float angleX = Dot( xAxis, normPos );
        float angleY = Dot( yAxis, normPos );

        mAngle.x = angleX;
        mAngle.y = angleY;

        mPosition = position;
    }
    else
    {
        mPosition = position;
    }
}

///---------------------------------------------------------------------
///<summary>
///J̕ݒ肷
///</summary>
///<param name="direction">J̕ݒ肷</param>
///---------------------------------------------------------------------
void Camera::SetDirection( Vector3 direction )
{
    mDirection = direction;
}

///---------------------------------------------------------------------
///<summary>
///J̃AbvxNgݒ肷
///</summary>
///<param name="upvector">J̃AbvxNg</param>
///---------------------------------------------------------------------
void Camera::SetUpVector( Vector3 upvector )
{
    mUpVector = upvector;
}

///---------------------------------------------------------------------
///<summary>
///r[svZ
///</summary>
///---------------------------------------------------------------------
void Camera::Compute()
{
    switch( mType )
    {
        case CAMERA_TYPE_THIRD_PERSON:
            {
                // 3l̃J
                ComputeThirdPersonCamera();
            }
            break;

        case CAMERA_TYPE_FIRST_PERSON:
            {
                // 1l̃J
                ComputeFirstPersonCamera();
            }
            break;

        default:
            {
                /* DO_NOTHING */
            }
            break;
    }
}

///---------------------------------------------------------------------
///<summary>
///J^Cv擾
///</summary>
///<returns>J^Cvԋp</returns>
///---------------------------------------------------------------------
int Camera::GetType()
{
    return mType;
}

///---------------------------------------------------------------------
///<summary>
///Jʒu擾
///</summary>
///<returns>vZ̃Jʒuԋp</returns>
///---------------------------------------------------------------------
Vector3 Camera::GetPosition()
{
    return mPosition;
}

///---------------------------------------------------------------------
///<summary>
///J̕擾
///</summary>
///<returns>vZ̃Jԋp</returns>
///---------------------------------------------------------------------
Vector3 Camera::GetDirection()
{
    return mDirection;
}

///---------------------------------------------------------------------
///<summary>
///J̃AbvxNg擾
///</summary>
///<returns>vZ̃J̃AbvxNgԋp</returns>
///---------------------------------------------------------------------
Vector3 Camera::GetUpVector()
{
    return mUpVector;
}

///---------------------------------------------------------------------
///<summary>
///J̃r[s擾
///</summary>
///<returns>vZ̃J̃r[sԋp</returns>
///---------------------------------------------------------------------
Matrix Camera::GetView()
{
    return mView;
}

///---------------------------------------------------------------------
///<summary>
///T[hp[\JƂăr[svZ
///</summary>
///---------------------------------------------------------------------
void Camera::ComputeThirdPersonCamera()
{
    // ]
    if ( App::GetInstance()->IsMouseDown( App::MOUSE_BUTTON_LEFT ) )
    {
        // ꂽ̃J[\ʒu擾
        int cursorX = App::GetInstance()->GetCursorX();
        int cursorY = App::GetInstance()->GetCursorY();

        // hbOłȂꍇ
        if ( !mLeft.isDrag )
        {
            // hbOtOON
            mLeft.isDrag = true;

            // ŏɉꂽƂ́CKv.
            mLeft.cursorX = cursorX;
            mLeft.cursorY = cursorY;
        }

        // vZ
        int deltaX = cursorX - mLeft.cursorX;
        int deltaY = cursorY - mLeft.cursorY;

        // XV
        mLeft.cursorX = cursorX;
        mLeft.cursorY = cursorY;

        // pxvZ
        mAngle.x += ToRadian( static_cast< float >( deltaX ) );
        mAngle.y += ToRadian( static_cast< float >( deltaY ) );

        // 0~360Ɏ߂
        if ( mAngle.x > 2.0f * Math::Pi )
        { mAngle.x -= ( 2.0f * Math::Pi ); }
        else if ( mAngle.x < 0.0f )
        { mAngle.x += ( 2.0f * Math::Pi ); }

        if ( mAngle.y > 2.0f * Math::Pi )
        { mAngle.y -= ( 2.0f * Math::Pi ); }
        else if ( mAngle.y < 0.0f )
        { mAngle.y += ( 2.0f * Math::Pi ); }
    }
    else if ( App::GetInstance()->IsMouseUp( App::MOUSE_BUTTON_LEFT ) )
    {
        // hbOtOOFF
        mLeft.isDrag = false;
    }
    
    // Y[
    if ( App::GetInstance()->IsMouseDown( App::MOUSE_BUTTON_RIGHT ) )
    {
        // ꂽ̃J[\ʒu擾
        int cursorX = App::GetInstance()->GetCursorX();
        int cursorY = App::GetInstance()->GetCursorY();

        // hbOłȂꍇ
        if ( !mRight.isDrag )
        {
            // hbOtOON
            mRight.isDrag = true;

            // ŏɉꂽƂ́CKv
            mRight.cursorX = cursorX;
            mRight.cursorY = cursorY;
        }

        // vZ
        int deltaX = cursorX - mRight.cursorX;
        int deltaY = cursorY - mRight.cursorY;

        // XV
        mRight.cursorX = cursorX;
        mRight.cursorY = cursorY;

        // Y[Xs[hW
        const float speed = 0.1f;

        // Y[vZ
        mDistance += ( deltaY ) * speed;
        
        // 0ȉɂȂȂ悤ɃK[h
        if ( mDistance < Math::F_Epsilon )
        { mDistance = Math::F_Epsilon; }
    }
    else if ( App::GetInstance()->IsMouseUp( App::MOUSE_BUTTON_RIGHT ) )
    {
        // hbOtOOFF
        mRight.isDrag = false;
    }

    // px
    mPosition.x = sinf( mAngle.x ) * cosf( mAngle.y );
    mPosition.y = sinf( mAngle.y );
    mPosition.z = cosf( mAngle.x ) * cosf( mAngle.y );

    // ʒu
    mPosition *= mDistance;

    // AbvxNgvZ
    if ( mAngle.y >= ToRadian( 90.0f ) && mAngle.y <= ToRadian( 270.0f ) )
    { mUpVector = Vector3( 0.0f, -1.0f, 0.0f ); }
    else
    { mUpVector = Vector3( 0.0f, 1.0f, 0.0f ); }

    // r[svZ
    mView = CreateLookAt( mPosition, mDirection, mUpVector );
}

///---------------------------------------------------------------------
///<summary>
///t@[Xgp[\JƂăr[svZ
///</summary>
///---------------------------------------------------------------------
void Camera::ComputeFirstPersonCamera()
{
    /* NOT_IMPLEMENTED */
}


} // namespace Graphics
} // namespace Asura
